"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
# ruins-ts

Ruins-ts is a library that converts TypeScript [fp-ts](https://github.com/gcanti/fp-ts) types into primitive types familiar to most JavaScript programmers. Using ruins-ts is generally speaking a _bad idea_ since the corresponding fp-ts types give much stronger guarantees about the values than their primitive JavaScript counterparts. However, if you wish you had a sledge hammer for breaking your application into smithereens, ruins-ts may be what you are looking for.

Ruins-ts was originally created as a convenience wrapper for dealing with [io-ts](https://github.com/gcanti/io-ts) codec return values. However, the [io-ts-validator](https://github.com/aeirola/io-ts-validator) has long ago surpassed ruins-ts for validation use. The ruins-ts is maintained by the time being for adhoc use, where writing a specific convenience library may not be feasible. If you have managed to read this far you may start using ruins-ts by importing it as follows.

*/
var ruins = __importStar(require("./ruins"));
ruins.fromOption; // terminates Option, returns nullable
ruins.fromEither; // terminates Either, throws on error
ruins.fromThese; // terminates These, logs warnings to console
ruins.fromIO; // terminates IO, returns the result
ruins.fromIOEither; // terminates IOEither, throws on error
ruins.fromTask; // terminates Task, returns a promise
ruins.fromTaskOption; // terminates TaskOption, resolves to nullable
ruins.fromTaskEither; // terminates TaskEither, rejects with error
ruins.fromTaskThese; // terminates TaskThese, logs warnings to console
var Option_ = __importStar(require("fp-ts/lib/Option"));
var someNumber = Option_.some(123);
var noNumber = Option_.none;
ruins.fromOption(someNumber); // => 123
ruins.fromOption(noNumber); // => null
var stringValue = Option_.some('foo');
var nullValue = Option_.some(null);
var noValue = Option_.none;
ruins.fromOption(stringValue); // => 'foo'
ruins.fromOption(nullValue); // => null
ruins.fromOption(noValue); // => null
nullValue === noValue; // => false
ruins.fromOption(nullValue) === ruins.fromOption(noValue); // => true
var Either_ = __importStar(require("fp-ts/lib/Either"));
var failureE = Either_.left('error');
var successE = Either_.right(123);
ruins.fromEither(failureE); // => never (throws an exception)
ruins.fromEither(successE); // => 123
var These_ = __importStar(require("fp-ts/lib/These"));
var failureT = These_.left('error');
var successT = These_.right(123);
var neutralT = These_.both('warning', 456);
ruins.fromThese(failureT); // => never (throws an exception)
ruins.fromThese(successT); // => 123
ruins.fromThese(neutralT); // => 456 (logs warning)
var syncComputation = function () {
    console.log('effect');
    return 123;
};
ruins.fromIO(syncComputation); // => 123 (prints "effect")
var failureIOE = function () {
    console.log('effect');
    return Either_.left('error');
};
var successIOE = function () {
    console.log('effect');
    return Either_.right(123);
};
ruins.fromIOEither(failureIOE); // => never (prints "effect", throws an exception)
ruins.fromIOEither(successIOE); // => 123 (prints "effect")
var asyncComputation = function () {
    return new Promise(function (resolve) {
        console.log('effect');
        resolve(123);
    });
};
ruins.fromTask(asyncComputation); // => Promise (prints "effect", resolves to 123)
var failureTO = function () {
    return new Promise(function (_resolve, reject) {
        console.log('effect');
        reject(Option_.none);
    });
};
var successTO = function () {
    return new Promise(function (resolve, _reject) {
        console.log('effect');
        resolve(Option_.some(123));
    });
};
ruins.fromTaskOption(failureTO); // => Promise (prints "effect", resolves to null)
ruins.fromTaskOption(successTO); // => Promise (prints "effect", resolves to 123)
var failureTE = function () {
    return new Promise(function (_resolve, reject) {
        console.log('effect');
        reject(Either_.left('error'));
    });
};
var successTE = function () {
    return new Promise(function (resolve, _reject) {
        console.log('effect');
        resolve(Either_.right(123));
    });
};
ruins.fromTaskEither(failureTE); // => Promise (prints "effect", rejects)
ruins.fromTaskEither(successTE); // => Promise (prints "effect", resolves to 123)
var failureTT = function () {
    return new Promise(function (_resolve, reject) {
        console.log('effect');
        reject(These_.left('error'));
    });
};
var successTT = function () {
    return new Promise(function (resolve, _reject) {
        console.log('effect');
        resolve(These_.right(123));
    });
};
var neutralTT = function () {
    return new Promise(function (resolve, _reject) {
        console.log('effect');
        resolve(These_.both('warning', 456));
    });
};
ruins.fromTaskThese(failureTT); // => Promise (prints "effect", rejects)
ruins.fromTaskThese(successTT); // => Promise (prints "effect", resolves to 123)
ruins.fromTaskThese(neutralTT); // => Promise (prints "effect", logs warning, resolves to 456)
/*
*/ 
